function [ varargout ] = avw_view_truecolor(avw,parent,command),

% avw_view_truecolor - Create and navigate ortho views of Analyze file
% 
% [ avw ] = avw_view_truecolor(avw, [parent], [command])
% 
% avw     - a struct, created by avw_img_read
%
% This variable can be any workspace variable name and return 
% values will be updated after each GUI calculation
% 
% parent  - an optional handle to the gui that calls this gui, 
% useful for updating the UserData field of the parent. The 
% avw structure may be returned to the parent, if possible.
% 
% command - an optional string, such as 'init' or various 
% callback commands, init is the default.
% 
% The navigation is by sliders and mouse clicks on the
% images in any of the ortho views.
% 
% Fiducial points can be selected, which are returned
% into 'avw.fiducials' in the base workspace. These points 
% are given in several coordinate frameworks (voxels, mm,
% and meters), with values given relative to an origin
% located at the "center" of the MRI volume (it is actually
% the bottom left corner of the central voxel, not the
% center of that voxel).
% 
% The AC location can be selected and the values are returned
% into 'avw.ac' in the base workspace. These points are given 
% in voxels, mm & meters; for the latter, the values are given
% as offsets from the "center" of the MRI volume.
% 
% +X is left (L), +Y is anterior (A), +Z is superior (S),
% the default LAS orientation of Analyze MRI files.  This
% is the radiological convention, as opposed to the
% neurological convention (RAS).
% 
% Example of loading and viewing the SPM T1 template:
% avw = avw_read('T1')
% avw = avw_view_truecolor(avw);
% 
% See also, avw_read, avw_img_read, avw_hdr_read
% 

% $Revision: 1.2 $ $Date: 2005/06/13 20:24:09 $

% Licence:  GNU GPL, no express or implied warranties
% History:  06/2002, Darren.Weber@flinders.edu.au
%           10/2002, Darren.Weber@flinders.edu.au
%                    added fiducial point determination
%                    changed plots from surf to imagesc commands
%                    added handling of datatype for avw.img
%                    altered daspect to use avw.hdr.dime.pixdim
%                    altered color scheme
%           01/2003, Darren.Weber@flinders.edu.au
%                    added parent GUI handling
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if ~exist('command','var'), command = 'init'; end

command = lower(command);

switch command,
case 'init',
    if(length(size(avw.img))==4)
        if(max(avw.img(:))>1)
            avw.img=avw.img/255;
        end
    end
    if ~exist('avw','var'),
        msg = sprintf('...no input avw - see help avw_view_truecolor\n');
        error(msg);
    end
    
    if isempty(inputname(1)),
        AVWVIEW = init(avw);
    else
        AVWVIEW = init(avw,inputname(1));
    end
    
    AVWVIEW = set_coordinates(AVWVIEW);
    
otherwise,
    AVWVIEW = get(gcbf,'Userdata');
end


switch command,

case {'axial_image','coronal_image','sagittal_image'},
    
    switch command,
    case 'axial_image',     view = 'axi'; axi = 0; cor = 1; sag = 1;
    case 'coronal_image',   view = 'cor'; axi = 1; cor = 0; sag = 1;
    case 'sagittal_image',  view = 'sag'; axi = 1; cor = 1; sag = 0;
    end
    AVWVIEW = get_slice_position(AVWVIEW,view);
    
    if axi,
        if isfield(AVWVIEW.handles,'axial_image'),
            Saxi = squeeze(AVWVIEW.avw.img(:,:,AVWVIEW.slices.axi,:));
            set(AVWVIEW.handles.axial_image,'CData',permute(Saxi,[2 1 3]));
        end
        if isfield(AVWVIEW.handles,'axial_sliderN'),
            set(AVWVIEW.handles.axial_sliderN,'String',num2str(AVWVIEW.slices.axi));
            set(AVWVIEW.handles.axial_sliderN,'Value',AVWVIEW.slices.axi);
        end
        if isfield(AVWVIEW.handles,'axial_slider'),
            set(AVWVIEW.handles.axial_slider,'Value',AVWVIEW.slices.axi);
        end;
    end
    
    if cor,
        if isfield(AVWVIEW.handles,'coronal_image'),
            Scor = squeeze(AVWVIEW.avw.img(:,AVWVIEW.slices.cor,:,:));
            set(AVWVIEW.handles.coronal_image,'CData',permute(Scor,[2 1 3]));
        end
        if isfield(AVWVIEW.handles,'coronal_sliderN'),
            set(AVWVIEW.handles.coronal_sliderN,'String',num2str(AVWVIEW.slices.cor));
            set(AVWVIEW.handles.coronal_sliderN,'Value',AVWVIEW.slices.cor);
        end
        if isfield(AVWVIEW.handles,'coronal_slider'),
            set(AVWVIEW.handles.coronal_slider,'Value',AVWVIEW.slices.cor);
        end;
    end;
    
    if sag,
        if isfield(AVWVIEW.handles,'sagittal_image'),
            Ssag = squeeze(AVWVIEW.avw.img(AVWVIEW.slices.sag,:,:,:));
            set(AVWVIEW.handles.sagittal_image,'CData',permute(Ssag,[2 1 3]));
        end
        if isfield(AVWVIEW.handles,'sagittal_sliderN'),
            set(AVWVIEW.handles.sagittal_sliderN,'String',num2str(AVWVIEW.slices.sag));
            set(AVWVIEW.handles.sagittal_sliderN,'Value',AVWVIEW.slices.sag);
        end
        if isfield(AVWVIEW.handles,'sagittal_slider'),
            set(AVWVIEW.handles.sagittal_slider,'Value',AVWVIEW.slices.sag);
        end;
    end;
    
    
case {'axial_slider','coronal_slider','sagittal_slider'},
    
    AVWVIEW = get_slider_position(AVWVIEW);
    
    switch command,
    case 'axial_slider',    axi = 1; cor = 0; sag = 0;
    case 'coronal_slider',  axi = 0; cor = 1; sag = 0;
    case 'sagittal_slider', axi = 0; cor = 0; sag = 1;
    end
    
    if axi,
        set(AVWVIEW.handles.axial_slider,'Value',AVWVIEW.slices.axi);
        Saxi = squeeze(AVWVIEW.avw.img(:,:,AVWVIEW.slices.axi,:));
        set(AVWVIEW.handles.axial_image,'CData',permute(Saxi,[2 1 3]));
        set(AVWVIEW.handles.axial_sliderN,'String',num2str(AVWVIEW.slices.axi));
        set(AVWVIEW.handles.axial_sliderN,'Value',AVWVIEW.slices.axi);
    end
    if cor,
        set(AVWVIEW.handles.coronal_slider,'Value',AVWVIEW.slices.cor);
        Scor = squeeze(AVWVIEW.avw.img(:,AVWVIEW.slices.cor,:,:));
        set(AVWVIEW.handles.coronal_image,'CData',permute(Scor,[2 1 3])); drawnow;
        set(AVWVIEW.handles.coronal_sliderN,'String',num2str(AVWVIEW.slices.cor));
        set(AVWVIEW.handles.coronal_sliderN,'Value',AVWVIEW.slices.cor);
    end    
    if sag,    
        set(AVWVIEW.handles.sagittal_slider,'Value',AVWVIEW.slices.sag);
        Ssag = squeeze(AVWVIEW.avw.img(AVWVIEW.slices.sag,:,:,:));
        set(AVWVIEW.handles.sagittal_image,'CData',permute(Ssag,[2 1 3])); drawnow;
        set(AVWVIEW.handles.sagittal_sliderN,'String',num2str(AVWVIEW.slices.sag));
        set(AVWVIEW.handles.sagittal_sliderN,'Value',AVWVIEW.slices.sag);
    end
    
    
case {'nasion','rpa','lpa','ac'},
    
    % return voxel coordinates into AVWVIEW.imgXYZ
    AVWVIEW = slices2metric(AVWVIEW);
        
    if get(AVWVIEW.handles.coord,'value') == 2,
        format = '%7.3f %7.3f %7.3f';
        imgXYZ = AVWVIEW.imgXYZ.mm;
        string = sprintf('%7.3f %7.3f %7.3f',imgXYZ);
    elseif get(AVWVIEW.handles.coord,'value') == 3,
        format = '%7.3f %7.3f %7.3f';
        imgXYZ = AVWVIEW.imgXYZ.meters;
        string = sprintf('%7.3f %7.3f %7.3f',imgXYZ);
    else
        imgXYZ = AVWVIEW.imgXYZ.voxels;
        string = sprintf('%7.0f %7.0f %7.0f',imgXYZ);
    end;
    
    switch command,
    case 'nasion',
        set(AVWVIEW.handles.nasion,'String',string);
        AVWVIEW.avw.fiducials.voxels(1,:) = AVWVIEW.imgXYZ.voxels;
        AVWVIEW.avw.fiducials.meters(1,:) = AVWVIEW.imgXYZ.meters;
        AVWVIEW.avw.fiducials.mm(1,:)     = AVWVIEW.imgXYZ.mm;
    case 'rpa',
        set(AVWVIEW.handles.rpa,'String',string);
        AVWVIEW.avw.fiducials.voxels(2,:) = AVWVIEW.imgXYZ.voxels;
        AVWVIEW.avw.fiducials.meters(2,:) = AVWVIEW.imgXYZ.meters;
        AVWVIEW.avw.fiducials.mm(2,:)     = AVWVIEW.imgXYZ.mm;
    case 'lpa',
        set(AVWVIEW.handles.lpa,'String',string);
        AVWVIEW.avw.fiducials.voxels(3,:) = AVWVIEW.imgXYZ.voxels;
        AVWVIEW.avw.fiducials.meters(3,:) = AVWVIEW.imgXYZ.meters;
        AVWVIEW.avw.fiducials.mm(3,:)     = AVWVIEW.imgXYZ.mm;
    case 'ac',
        set(AVWVIEW.handles.ac,'String',string);
        AVWVIEW.avw.ac.voxels = AVWVIEW.imgXYZ.voxels;
        AVWVIEW.avw.ac.meters = AVWVIEW.imgXYZ.meters;
        AVWVIEW.avw.ac.mm = AVWVIEW.imgXYZ.mm;
    end
    
    
case 'flip',
    
    xdim = size(AVWVIEW.avw.img,1);
    AVWVIEW.avw.img = AVWVIEW.avw.img(xdim:-1:1,:,:,:);
    
    if isfield(AVWVIEW.handles,'axial_image'),
        Saxi = squeeze(AVWVIEW.avw.img(:,:,AVWVIEW.slices.axi,:));
        set(AVWVIEW.handles.axial_image,'CData',permute(Saxi,[2 1 3]));
    end;
    if isfield(AVWVIEW.handles,'coronal_image'),
        Scor = squeeze(AVWVIEW.avw.img(:,AVWVIEW.slices.cor,:,:));
        set(AVWVIEW.handles.coronal_image,'CData',permute(Scor,[2 1 3]));
    end;
    if isfield(AVWVIEW.handles,'sagittal_image'),
        Ssag = squeeze(AVWVIEW.avw.img(AVWVIEW.slices.sag,:,:,:));
        set(AVWVIEW.handles.sagittal_image,'CData',permute(Ssag,[2 1 3]));
    end;
    
    flipStatus = get(AVWVIEW.handles.flipStatus,'string');
    if strmatch(flipStatus,'orig'),
        flipStatus = 'flipped';
    else,
        flipStatus = 'orig';
    end;
    set(AVWVIEW.handles.flipStatus,'string',flipStatus);
    
    
case 'coordinates',
    
    AVWVIEW = set_coordinates(AVWVIEW);
    
case {'brighter','dimmer'},
    
    switch command,     
    case 'brighter', level = 0.9;
        if(strcmp('uint8',class(AVWVIEW.avw.img))) 
            AVWVIEW.avw.img = uint8(min(255,1.2*double(AVWVIEW.avw.img)));
        else
            AVWVIEW.avw.img = (min(1,1.2*(AVWVIEW.avw.img)));            
        end
    case 'dimmer', level = 1.1;
        if(strcmp('uint8',class(AVWVIEW.avw.img))) 
            AVWVIEW.avw.img = uint8(.9*double(AVWVIEW.avw.img));
        else
            AVWVIEW.avw.img = .9*(AVWVIEW.avw.img);  
        end        
    end
%    set(gcbf,'Userdata','AVWVIEW');
        set(AVWVIEW.gui,'UserData',AVWVIEW);
    avw_view_truecolor([],[],'axial_image');
    avw_view_truecolor([],[],'coronal_image');
    avw_view_truecolor([],[],'sagittal_image');

    
        
%     if isfield(AVWVIEW.handles,'axial_image'),
%         clim = get(AVWVIEW.handles.axial_axes,'clim');
%         set(AVWVIEW.handles.axial_axes,'Clim',double(clim) .* level);
%     end;
%     if isfield(AVWVIEW.handles,'coronal_image'),
%         clim = get(AVWVIEW.handles.coronal_axes,'clim');
%         set(AVWVIEW.handles.coronal_axes,'Clim',double(clim) .* level);
%     end;
%     if isfield(AVWVIEW.handles,'sagittal_image'),
%         clim = get(AVWVIEW.handles.sagittal_axes,'clim');
%         set(AVWVIEW.handles.sagittal_axes,'Clim',double(clim) .* level);
%     end;
    
case 'histogram',
    
    avw = AVWVIEW.avw;
    
    % would be nice to use bins that reflect
    % the bits per pixel, but it seems to take
    % forever for a 16 bit image, otherwise
    % use this code:
    %     %check the bits per pixel of avw
    %     bitpix = avw.hdr.dime.bitpix;
    %     % set the bins according to the data type
    %     if bitpix <= 8, bins = 0:255; end
    %     if bitpix > 8,  bins = 0:65535; end
    
    %bins = linspace(0,intensity_max,255);
    
    intensity_min = double(min(min(min(avw.img(:)))));
    intensity_max = double(max(max(max(avw.img(:)))));
    
    bins = intensity_min:intensity_max;
    
    fprintf('...calculating histogram for %d bins.\n',length(bins));
    
    for i=1:size(avw.img,3),
        n = hist(double(avw.img(:,:,i)), bins);
        if i == 1,
            intensity_volume = sum(n,2);
        else
            intensity_slice = sum(n,2);
            intensity_volume = intensity_volume + intensity_slice;
        end
    end
    figure('name','intensity histogram'); bar(bins,intensity_volume);
    
    
case 'quit',
    
    % try to assign back into the input struct
    if ~isempty(AVWVIEW.invarname),
        fprintf('...returning data into base workspace struct ''%s''\n',AVWVIEW.invarname);
        assignin('base',AVWVIEW.invarname,AVWVIEW.avw);
    elseif evalin('base','exist(''mri'',''var'')'),
        fprintf('...returning data into base workspace struct ''mri''\n');
        evalin('base',['AVWVIEW = get(', num2str(AVWVIEW.gui),',''Userdata''); mri.data = AVWVIEW.avw; clear AVWVIEW;']);
    else
        fprintf('...returning data into base workspace struct ''avw''\n');
        assignin('base','avw',AVWVIEW.avw);
    end
    close gcbf;
    
otherwise,
    
end


switch command,
case 'quit',
otherwise,
    set(AVWVIEW.gui,'UserData',AVWVIEW);
end

if nargout > 0,
    AVWVIEW.avw.img = double(AVWVIEW.avw.img);
    varargout{1} = AVWVIEW.avw;
end


return






















%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function AVWVIEW = set_coordinates(AVWVIEW)
    
    % set the axis coordinates to voxels, mm or meters
    
    s = size(AVWVIEW.avw.img);
    if length(s) > 0, xdim = s(1); else xdim = 0; end
    if length(s) > 1, ydim = s(2); else ydim = 0; end
    if length(s) > 2, zdim = s(3); else zdim = 0; end
    
    if get(AVWVIEW.handles.coord,'value') == 2,         % mm
        xpixdim = double(AVWVIEW.avw.hdr.dime.pixdim(2));
        ypixdim = double(AVWVIEW.avw.hdr.dime.pixdim(3));
        zpixdim = double(AVWVIEW.avw.hdr.dime.pixdim(4));
        aspect = [1 1 1];
        xdata = [0 (xdim * xpixdim)];
        ydata = [0 (ydim * ypixdim)];
        zdata = [0 (zdim * zpixdim)];
                
    elseif get(AVWVIEW.handles.coord,'value') == 3,     % meters
        xpixdim = double(AVWVIEW.avw.hdr.dime.pixdim(2)) / 1000;
        ypixdim = double(AVWVIEW.avw.hdr.dime.pixdim(3)) / 1000;
        zpixdim = double(AVWVIEW.avw.hdr.dime.pixdim(4)) / 1000;
        aspect = [1 1 1];
        xdata = [0 (xdim * xpixdim)];
        ydata = [0 (ydim * ypixdim)];
        zdata = [0 (zdim * zpixdim)];
        
    else,                                               % voxels
        xpixdim = double(AVWVIEW.avw.hdr.dime.pixdim(2));
        ypixdim = double(AVWVIEW.avw.hdr.dime.pixdim(3));
        zpixdim = double(AVWVIEW.avw.hdr.dime.pixdim(4));
        aspect = AVWVIEW.avw.hdr.dime.pixdim(2:4);
        xdata = [0 xdim];
        ydata = [0 ydim];
        zdata = [0 zdim];
    end;
    
aspect = 1./aspect; %AVWVIEW.avw.hdr.dime.pixdim(2:4);

    
    if isfield(AVWVIEW.handles,'axial_image'),
        set(AVWVIEW.handles.axial_axes,'Xlim',xdata);
        set(AVWVIEW.handles.axial_axes,'Ylim',ydata);
        set(AVWVIEW.handles.axial_axes,'Zlim',zdata);
        set(AVWVIEW.handles.axial_image,'Xdata',xdata);
        set(AVWVIEW.handles.axial_image,'Ydata',ydata);
        daspect(AVWVIEW.handles.axial_axes,aspect([1 2 3]));
    end;
    if isfield(AVWVIEW.handles,'coronal_image'),
        set(AVWVIEW.handles.coronal_axes,'Xlim',xdata);
        set(AVWVIEW.handles.coronal_axes,'Ylim',zdata);
        set(AVWVIEW.handles.coronal_axes,'Zlim',ydata);
        set(AVWVIEW.handles.coronal_image,'Xdata',xdata);
        set(AVWVIEW.handles.coronal_image,'Ydata',zdata);
        daspect(AVWVIEW.handles.coronal_axes,aspect([1 3 2]));
    end;
    if isfield(AVWVIEW.handles,'sagittal_image'),
        set(AVWVIEW.handles.sagittal_axes,'Xlim',ydata);
        set(AVWVIEW.handles.sagittal_axes,'Ylim',zdata);
        set(AVWVIEW.handles.sagittal_axes,'Zlim',xdata);
        set(AVWVIEW.handles.sagittal_image,'Xdata',ydata);
        set(AVWVIEW.handles.sagittal_image,'Ydata',zdata);
        daspect(AVWVIEW.handles.sagittal_axes,aspect([2 3 1]));
    end;
    
    set_image_value(AVWVIEW);
    
return




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function AVWVIEW = get_slice_position(AVWVIEW,view),
	
	AVWVIEW = get_slider_position(AVWVIEW);
	
    [AVWVIEW, slices] = slices2metric(AVWVIEW);
	
    switch view,
    case 'sag',
        currentpoint = get(get(AVWVIEW.handles.sagittal_image,'Parent'),'CurrentPoint');
        slices.cor = currentpoint(1,1);
        slices.axi = currentpoint(1,2);
    case 'cor',
        currentpoint = get(get(AVWVIEW.handles.coronal_image,'Parent'),'CurrentPoint');
        slices.sag = currentpoint(2,1);
        slices.axi = currentpoint(2,2);
    case 'axi',
        currentpoint = get(get(AVWVIEW.handles.axial_image,'Parent'),'CurrentPoint');
        slices.sag = currentpoint(2,1);
        slices.cor = currentpoint(2,2);
    end
    
    AVWVIEW = metric2slices(AVWVIEW,slices);
    
    AVWVIEW = check_slices(AVWVIEW);
    
    set_image_value(AVWVIEW);
    
return


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function AVWVIEW = get_slider_position(AVWVIEW),
    
    [AVWVIEW.slices.sag,AVWVIEW.slices.cor,AVWVIEW.slices.axi] = deal(0);
    
    if isfield(AVWVIEW.handles,'sagittal_slider'),
        if ishandle(AVWVIEW.handles.sagittal_slider),
            AVWVIEW.slices.sag = round(get(AVWVIEW.handles.sagittal_slider,'Value'));
        end
    end
    if AVWVIEW.slices.sag == 0,
        if isfield(AVWVIEW.handles,'sagittal_sliderN'),
            if ishandle(AVWVIEW.handles.sagittal_sliderN),
                AVWVIEW.slices.sag = round(get(AVWVIEW.handles.sagittal_sliderN,'Value'));
            end
        end
    end
    
    
    if isfield(AVWVIEW.handles,'coronal_slider'),
        if ishandle(AVWVIEW.handles.coronal_slider),
            AVWVIEW.slices.cor = round(get(AVWVIEW.handles.coronal_slider,'Value'));
        end
    end
    if AVWVIEW.slices.cor == 0,
        if isfield(AVWVIEW.handles,'coronal_sliderN'),
            if ishandle(AVWVIEW.handles.coronal_sliderN),
                AVWVIEW.slices.cor = round(get(AVWVIEW.handles.coronal_sliderN,'Value'));
            end
        end
    end
    
    if isfield(AVWVIEW.handles,'axial_slider'),
        if ishandle(AVWVIEW.handles.axial_slider),
            AVWVIEW.slices.axi = round(get(AVWVIEW.handles.axial_slider,'Value'));
        end
    end
    if AVWVIEW.slices.axi == 0,
        if isfield(AVWVIEW.handles,'axial_sliderN'),
            if ishandle(AVWVIEW.handles.axial_sliderN),
                AVWVIEW.slices.axi = round(get(AVWVIEW.handles.axial_sliderN,'Value'));
            end
        end
    end
    
    AVWVIEW = check_slices(AVWVIEW);
    set_image_value(AVWVIEW);
    
return


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function AVWVIEW = check_slices(AVWVIEW),
    
    [ SagSize, CorSize, AxiSize ] = size(AVWVIEW.avw.img);
    if AVWVIEW.slices.sag > SagSize, AVWVIEW.slices.sag = SagSize; end;
    if AVWVIEW.slices.sag < 1, AVWVIEW.slices.sag = 1; end;
    if AVWVIEW.slices.cor > CorSize, AVWVIEW.slices.cor = CorSize; end;
    if AVWVIEW.slices.cor < 1, AVWVIEW.slices.cor = 1; end;
    if AVWVIEW.slices.axi > AxiSize, AVWVIEW.slices.axi = AxiSize; end;
    if AVWVIEW.slices.axi < 1, AVWVIEW.slices.axi = 1; end;
    
return


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function set_image_value(AVWVIEW),
    
    % get coordinates of selected voxel and the image intensity there
    
    sag = AVWVIEW.slices.sag;
    cor = AVWVIEW.slices.cor;
    axi = AVWVIEW.slices.axi;
    
    img = AVWVIEW.avw.img;
    
    imgvalue = double(img(sag,cor,axi));
    
    set(AVWVIEW.handles.imval,'String',sprintf('%7.2f',imgvalue));
    set(AVWVIEW.handles.imval,'Value',imgvalue);
    
    % Now update the coordinates of the selected voxel
    
    [AVWVIEW, slices] = slices2metric(AVWVIEW);
    sag = slices.sag;
    cor = slices.cor;
    axi = slices.axi;
    
    if get(AVWVIEW.handles.coord,'value') == 2,
        string = sprintf('%7.3f %7.3f %7.3f',sag,cor,axi);
    elseif get(AVWVIEW.handles.coord,'value') == 3,
        string = sprintf('%7.3f %7.3f %7.3f',sag,cor,axi);
    else
        string = sprintf('%7.0f %7.0f %7.0f',sag,cor,axi);
    end;
    
    set(AVWVIEW.handles.impos,'String',string);
    set(AVWVIEW.handles.impos,'Value',[sag,cor,axi]);
    
return


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [AVWVIEW,slices] = slices2metric(AVWVIEW),
    
    AVWVIEW.imgXYZ.voxels = [AVWVIEW.slices.sag,AVWVIEW.slices.cor,AVWVIEW.slices.axi];
    AVWVIEW.imgXYZ.meters = (AVWVIEW.imgXYZ.voxels - AVWVIEW.center) .* AVWVIEW.scale2meters;
    AVWVIEW.imgXYZ.mm     = (AVWVIEW.imgXYZ.voxels - AVWVIEW.center) .* AVWVIEW.scale2mm;
    
    if get(AVWVIEW.handles.coord,'value') == 2,
        % using mm mensuration on axes
        xpix = double(AVWVIEW.avw.hdr.dime.pixdim(2));
        ypix = double(AVWVIEW.avw.hdr.dime.pixdim(3));
        zpix = double(AVWVIEW.avw.hdr.dime.pixdim(4));
        slices.axi = AVWVIEW.slices.axi * zpix;
        slices.cor = AVWVIEW.slices.cor * ypix;
        slices.sag = AVWVIEW.slices.sag * xpix;
    elseif get(AVWVIEW.handles.coord,'value') == 3,
        % using meters
        xpix = double(AVWVIEW.avw.hdr.dime.pixdim(2)) / 1000;
        ypix = double(AVWVIEW.avw.hdr.dime.pixdim(3)) / 1000;
        zpix = double(AVWVIEW.avw.hdr.dime.pixdim(4)) / 1000;
        slices.axi = AVWVIEW.slices.axi * zpix;
        slices.cor = AVWVIEW.slices.cor * ypix;
        slices.sag = AVWVIEW.slices.sag * xpix;
    else
        % voxels
        slices.axi = AVWVIEW.slices.axi;
        slices.cor = AVWVIEW.slices.cor;
        slices.sag = AVWVIEW.slices.sag;
    end;
    
return


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function AVWVIEW = metric2slices(AVWVIEW,slices),
    
    if get(AVWVIEW.handles.coord,'value') == 2,
        % using mm
        xpix = double(AVWVIEW.avw.hdr.dime.pixdim(2));
        ypix = double(AVWVIEW.avw.hdr.dime.pixdim(3));
        zpix = double(AVWVIEW.avw.hdr.dime.pixdim(4));
        AVWVIEW.slices.axi = round(slices.axi / zpix);
        AVWVIEW.slices.cor = round(slices.cor / ypix);
        AVWVIEW.slices.sag = round(slices.sag / xpix);
    elseif get(AVWVIEW.handles.coord,'value') == 3,
        % using meters
        xpix = double(AVWVIEW.avw.hdr.dime.pixdim(2)) / 1000;
        ypix = double(AVWVIEW.avw.hdr.dime.pixdim(3)) / 1000;
        zpix = double(AVWVIEW.avw.hdr.dime.pixdim(4)) / 1000;
        AVWVIEW.slices.axi = round(slices.axi / zpix);
        AVWVIEW.slices.cor = round(slices.cor / ypix);
        AVWVIEW.slices.sag = round(slices.sag / xpix);
    else
        % voxels
        AVWVIEW.slices.axi = round(slices.axi);
        AVWVIEW.slices.cor = round(slices.cor);
        AVWVIEW.slices.sag = round(slices.sag);
    end;
    
return



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function AVWVIEW = init(avw,invarname),
	
    % try to keep track of the input struct
	if exist('invarname','var'),
        if ~isempty(invarname),
            AVWVIEW.invarname = invarname;
        else
            AVWVIEW.invarname = '';
        end
	else
        AVWVIEW.invarname = '';
	end
	
	% GUI General Parameters
	GUIwidth  = 150;
	GUIheight = 50;
	
    version = '[$Revision: 1.2 $]';
    name = sprintf('AVW View [v%s]',version(12:16));
    
	if isfield(avw,'fileprefix'),
        if ~isempty(avw.fileprefix),
            format = strcat('%+',sprintf('%d',length(avw.fileprefix)+1),'s');
            name = strcat(name,' - ',sprintf(format,avw.fileprefix));
        end
	end
	
	GUI = figure('Name',name,'Tag','AVWVIEW','units','characters',...
        'NumberTitle','off','color',[0 0 0],...
        'MenuBar','figure','Position',[1 1 GUIwidth GUIheight]);
	movegui(GUI,'center');
	
	AVWVIEW.gui = GUI;
	
	Font.FontName   = 'Helvetica';
	Font.FontUnits  = 'Pixels';
	Font.FontSize   = 9;
	Font.FontWeight = 'normal';
	Font.FontAngle  = 'normal';
	
	
	shading flat
	
	
	
% 	% determine the datatype of avw.img
% 	switch double(avw.hdr.dime.bitpix),
% 	case 1,
%         fprintf('...converting avw.img to uint8 for viewing only.\n\n');
%         avw.img = uint8(avw.img);
% 	case 8,
%         fprintf('...converting avw.img to uint8 for viewing only.\n\n');
%         avw.img = uint8(avw.img);
% 	case 16,
%         fprintf('...converting avw.img to uint16 for viewing only.\n\n');
%         avw.img = uint16(avw.img);
% 	case {32,64},
%         % make sure it is double, not single
%         avw.img = double(avw.img);
% 	otherwise,
%         % do nothing, leave it as is
% 	end
    
    
    % calculate image stats
    intensityMean = mean(mean(mean(double(avw.img))));
    intensityMeanRobust = mean(mean(mean(double(avw.img(find(avw.img))))));
    intensityStdev = 0; %std(std(std(double(avw.img))));
    
    intensityMax = max(max(max(double(avw.img))));
    
    % try to adjust for large intensity extremes
    if intensityMax > 255,
        % got 16 bit data, at least
        adjust = 0.5;
    else
        adjust = 0.9;
    end
	%clim = [0 (intensityMeanRobust + (10 * intensityStdev)) ];
%	clim = [0 (intensityMax * adjust)];
	
	
	
	xdim = size(avw.img,1);
	ydim = size(avw.img,2);
	zdim = size(avw.img,3);
	
	AVWVIEW.slices.sag = 1;
	AVWVIEW.slices.cor = 1;
	AVWVIEW.slices.axi = 1;
	if xdim > 1, AVWVIEW.slices.sag = floor(xdim/2); end
	if ydim > 1, AVWVIEW.slices.cor = floor(ydim/2); end
	if zdim > 1, AVWVIEW.slices.axi = floor(zdim/2); end
	
	% store the volume center for later reference when
	% calculating fiducial locations
	AVWVIEW.center  = [AVWVIEW.slices.sag,AVWVIEW.slices.cor,AVWVIEW.slices.axi];
	
	% set the default origin at the center
	AVWVIEW.origin  = AVWVIEW.center;
	
	
	pixdim = double(avw.hdr.dime.pixdim(2:4));
	
    AVWVIEW.scale2mm     = pixdim;          % vol scale in mm
	AVWVIEW.scale2meters = pixdim ./ 1000;  % vol scale in meters
	AVWVIEW.daspect      = pixdim ./ min(pixdim);
    
    xPlotSize = 0.38;
    yPlotSize = 0.38;
    
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	% Axial Slice
	if xdim > 1 & ydim > 1,
        
        handles.axial_subplot = subplot('position',[0.075 0.075 xPlotSize yPlotSize]);
        colormap('gray');
        
        Saxial = squeeze(avw.img(:,:,AVWVIEW.slices.axi,:));
        
        %handles.axial_image = pcolor(double(Saxial')); colormap(gray); shading interp
        %pcolor(Saxial'); colormap(gray); shading flat
        
        %surf(avw.img(:,:,20)','edgecolor','none'); view(2); axis tight; colormap(flipdim(gray,1)); colorbar
        %surf(avw.img(:,:,20)','edgecolor','none'); view(2); axis tight; colormap(gray)
        %image(avw.img(:,:,20)'); axis image; colormap(gray)
        %imagesc(avw.img(:,:,20)'); axis image; colormap(gray)
        
        %handles.axial_image = image('Cdata',Saxial','CDataMapping','scaled',...
        %    'XData',[0 xdim],'YData',[0 ydim]);
        
        handles.axial_image = imagesc([0,xdim],[0,ydim],permute(Saxial,[2 1 3]));
        set(gcf,'DoubleBuffer','on') % To turn on double buffering to reduce flicker

        handles.axial_axes = gca;
        set(gca,'YDir','normal','XLimMode','manual','YLimMode','manual',...
            'ClimMode','manual','YColor',[1 1 1],'XColor',[1 1 1])
        
        title('Axial','Color',[1 1 1])
        ylabel('Y')
        xlabel('X')
        %xlabel('(Right <<) X (>> Left)'); % default radiological orientation for Analyze
        %xlabel('(Left <<) X (>> Right)')
        
        % This callback navigates with left click
        set(handles.axial_image,'ButtonDownFcn','avw_view_truecolor([],[],''axial_image'');');
        
        
        if zdim > 1,
            slider_step(1) = 1/(zdim);
            slider_step(2) = 1/(zdim);
            handles.axial_slider = uicontrol('Parent',GUI,'Style','slider','Units','Normalized', Font, ...
                'Position',[.65 .14 .10 .03], 'HorizontalAlignment', 'center',...
                'BusyAction','queue',...
                'TooltipString','Axial slice navigation',...
                'Min',1,'Max',zdim,'SliderStep',slider_step,'Value',AVWVIEW.slices.axi,...
                'Callback','avw_view_truecolor([],[],''axial_slider'');');
        end
        handles.axial_sliderN = uicontrol('Parent',GUI,'Style','text','Units','Normalized', Font, ...
            'Position',[.75 .14 .03 .03], 'HorizontalAlignment', 'center',...
            'BackgroundColor',[0 0 0],'ForegroundColor',[1 1 1],...
            'BusyAction','queue',...
            'TooltipString','Axial slice number',...
            'String',num2str(AVWVIEW.slices.axi),'Value',AVWVIEW.slices.axi);
        handles.axial_sliderT = uicontrol('Parent',GUI,'Style','text','Units','Normalized', Font, ...
            'Position',[.55 .14 .10 .03], 'HorizontalAlignment', 'center',...
            'BackgroundColor',[0 0 0],'ForegroundColor',[1 1 1],...
            'BusyAction','queue',...
            'TooltipString','Axial slice navigation',...
            'String','Axial');
	end
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	% Coronal Slice
	if xdim > 1 & zdim > 1,
        
        handles.coronal_subplot = subplot('position',[0.075 0.575 xPlotSize yPlotSize]);
        colormap('gray');
        
        Scor = squeeze(avw.img(:,AVWVIEW.slices.cor,:,:));
        handles.coronal_image = imagesc([0,xdim],[0,zdim],permute(Scor,[2 1 3]));
        set(gcf,'DoubleBuffer','on') % To turn it on 

        handles.coronal_axes = gca;
        set(gca,'YDir','normal','XLimMode','manual','YLimMode','manual',...
            'ClimMode','manual','YColor',[1 1 1],'XColor',[1 1 1])
        
        %xlabel('(Left <<) X (>> Right)')
        %xlabel('(Right <<) X (>> Left)')
        xlabel('X')
        ylabel('Z')
        title('Coronal','Color',[1 1 1])
        
        % This callback navigates with left click
        set(handles.coronal_image,'ButtonDownFcn','avw_view_truecolor([],[],''coronal_image'');');
        
        if ydim > 1,
            slider_step(1) = 1/(ydim);
            slider_step(2) = 1/(ydim);
            handles.coronal_slider = uicontrol('Parent',GUI,'Style','slider','Units','Normalized', Font, ...
                'Position',[.65 .10 .10 .03], 'HorizontalAlignment', 'center',...
                'BusyAction','queue',...
                'TooltipString','Coronal slice navigation',...
                'Min',1,'Max',ydim,'SliderStep',slider_step,'Value',AVWVIEW.slices.cor,...
                'Callback','avw_view_truecolor([],[],''coronal_slider'');');
        end
        handles.coronal_sliderN = uicontrol('Parent',GUI,'Style','text','Units','Normalized', Font, ...
            'Position',[.75 .10 .03 .03], 'HorizontalAlignment', 'center',...
            'BackgroundColor',[0 0 0],'ForegroundColor',[1 1 1],...
            'BusyAction','queue',...
            'TooltipString','Coronal slice number',...
            'String',num2str(AVWVIEW.slices.cor),'Value',AVWVIEW.slices.cor);
        handles.coronal_sliderT = uicontrol('Parent',GUI,'Style','text','Units','Normalized', Font, ...
            'Position',[.55 .10 .10 .03], 'HorizontalAlignment', 'center',...
            'BackgroundColor',[0 0 0],'ForegroundColor',[1 1 1],...
            'BusyAction','queue',...
            'TooltipString','Coronal slice navigation',...
            'String','Coronal');
        
	end
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	% Sagittal Slice
	if ydim > 1 & zdim > 1,
        
        handles.sagittal_subplot = subplot('position',[0.575 0.575 xPlotSize yPlotSize]);
        colormap('gray');
        
        Ssag = squeeze(avw.img(AVWVIEW.slices.sag,:,:,:));
        handles.sagittal_image = imagesc([0,ydim],[0,zdim],permute(Ssag,[2 1 3]));
set(gcf,'DoubleBuffer','on') % To turn it on 
        
        handles.sagittal_axes = gca;
        set(gca,'YDir','normal','XLimMode','manual','YLimMode','manual',...
            'ClimMode','manual','YColor',[1 1 1],'XColor',[1 1 1])
        
        xlabel('Y')
        ylabel('Z')
        title('Sagittal','Color',[1 1 1])
        
        % This callback navigates with mouse click
        set(handles.sagittal_image,'ButtonDownFcn','avw_view_truecolor([],[],''sagittal_image'');');
        
        if xdim > 1,
            slider_step(1) = 1/(xdim);
            slider_step(2) = 1/(xdim);
            handles.sagittal_slider = uicontrol('Parent',GUI,'Style','slider','Units','Normalized', Font, ...
                'Position',[.65 .06 .10 .03], 'HorizontalAlignment', 'center',...
                'BusyAction','queue',...
                'TooltipString','Sagittal slice navigation',...
                'Min',1,'Max',xdim,'SliderStep',slider_step,'Value',AVWVIEW.slices.sag,...
                'Callback','avw_view_truecolor([],[],''sagittal_slider'');');
        end
        handles.sagittal_sliderN = uicontrol('Parent',GUI,'Style','text','Units','Normalized', Font, ...
            'Position',[.75 .06 .03 .03], 'HorizontalAlignment', 'center',...
            'BackgroundColor',[0 0 0],'ForegroundColor',[1 1 1],...
            'BusyAction','queue',...
            'TooltipString','Sagittal slice number',...
            'String',num2str(AVWVIEW.slices.sag),'Value',AVWVIEW.slices.sag);
        handles.sagittal_sliderT = uicontrol('Parent',GUI,'Style','text','Units','Normalized', Font, ...
            'Position',[.55 .06 .10 .03], 'HorizontalAlignment', 'center',...
            'BackgroundColor',[0 0 0],'ForegroundColor',[1 1 1],...
            'BusyAction','queue',...
            'TooltipString','Sagittal slice navigation',...
            'String','Sagittal');
	end
	
	
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	% Image Intensity at Mouse Click
	
	handles.Timval = uicontrol('Parent',GUI,'Style','text','Units','Normalized', Font, ...
        'Position',[.55 .46 .10 .03], 'HorizontalAlignment', 'left',...
        'BackgroundColor', [0 0 0],...
        'ForegroundColor', [1 1 1],...
        'BusyAction','queue',...
        'String','Image Intensity');
	handles.imval = uicontrol('Parent',GUI,'Style','text','Units','Normalized', Font, ...
        'Position',[.65 .46 .15 .03], 'HorizontalAlignment', 'right',...
        'BackgroundColor', [0 0 0],...
        'ForegroundColor', [1 1 1],...
        'BusyAction','queue',...
        'String','x','Value',0);
	
	% Image Position at Mouse Click
	
	handles.Timpos = uicontrol('Parent',GUI,'Style','text','Units','Normalized', Font, ...
        'Position',[.55 .42 .10 .03], 'HorizontalAlignment', 'left',...
        'BackgroundColor', [0 0 0],...
        'ForegroundColor', [1 1 1],...
        'BusyAction','queue',...
        'String','Image Position');
	handles.impos = uicontrol('Parent',GUI,'Style','text','Units','Normalized', Font, ...
        'Position',[.65 .42 .15 .03], 'HorizontalAlignment', 'right',...
        'BackgroundColor', [0 0 0],...
        'ForegroundColor', [1 1 1],...
        'BusyAction','queue',...
        'String','xyz','Value',[0 0 0]);
	
    
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	% AC Location
    
	handles.Tac = uicontrol('Parent',GUI,'Style','pushbutton','Units','Normalized', Font, ...
        'Position',[.55 .38 .10 .03], 'HorizontalAlignment', 'left',...
        'BackgroundColor', [.3 .3 .3],...
        'ForegroundColor', [1 1 1],...
        'BusyAction','queue',...
        'TooltipString','AC point in (voxels) or (meter offset from center of volume)',...
        'String','AC Point',...
        'Callback','avw_view_truecolor([],[],''ac'');');
	handles.ac = uicontrol('Parent',GUI,'Style','text','Units','Normalized', Font, ...
        'Position',[.65 .38 .15 .03], 'HorizontalAlignment', 'right',...
        'BackgroundColor', [0 0 0],...
        'ForegroundColor', [1 1 1],...
        'BusyAction','queue',...
        'TooltipString','These values are offset from volume center.',...
        'String','x,y,z');
	
	% Nasion Location
    
	handles.Tnasion = uicontrol('Parent',GUI,'Style','pushbutton','Units','Normalized', Font, ...
        'Position',[.55 .34 .10 .03], 'HorizontalAlignment', 'left',...
        'BackgroundColor', [.3 .3 .3],...
        'ForegroundColor', [1 1 1],...
        'BusyAction','queue',...
        'TooltipString','Update Nasion - should be toward +Y',...
        'String','Fiducial: Nas',...
        'Callback','avw_view_truecolor([],[],''nasion'');');
	handles.nasion = uicontrol('Parent',GUI,'Style','text','Units','Normalized', Font, ...
        'Position',[.65 .34 .15 .03], 'HorizontalAlignment', 'right',...
        'BackgroundColor', [0 0 0],...
        'ForegroundColor', [1 1 1],...
        'BusyAction','queue',...
        'TooltipString','These values are offset from volume center, should be toward +Y',...
        'String','x,y,z');
	
	% Right Preauricular Location
    
	handles.Trpa = uicontrol('Parent',GUI,'Style','pushbutton','Units','Normalized', Font, ...
        'Position',[.55 .30 .10 .03], 'HorizontalAlignment', 'left',...
        'BackgroundColor', [.3 .3 .3],...
        'ForegroundColor', [1 1 1],...
        'BusyAction','queue',...
        'TooltipString','Update Right Preauricular - should be toward -X',...
        'String','Fiducial: RPA',...
        'Callback','avw_view_truecolor([],[],''rpa'');');
	handles.rpa = uicontrol('Parent',GUI,'Style','text','Units','Normalized', Font, ...
        'Position',[.65 .30 .15 .03], 'HorizontalAlignment', 'right',...
        'BackgroundColor', [0 0 0],...
        'ForegroundColor', [1 1 1],...
        'BusyAction','queue',...
        'TooltipString','These values are offset from volume center, should be toward -X',...
        'String','x,y,z');
	
	% Left Preauricular Location
    
	handles.Tlpa = uicontrol('Parent',GUI,'Style','pushbutton','Units','Normalized', Font, ...
        'Position',[.55 .26 .10 .03], 'HorizontalAlignment', 'left',...
        'BackgroundColor', [.3 .3 .3],...
        'ForegroundColor', [1 1 1],...
        'BusyAction','queue',...
        'TooltipString','Update Left Preauricular - should be toward +X',...
        'String','Fiducial: LPA',...
        'Callback','avw_view_truecolor([],[],''lpa'');');
	handles.lpa = uicontrol('Parent',GUI,'Style','text','Units','Normalized', Font, ...
        'Position',[.65 .26 .15 .03], 'HorizontalAlignment', 'right',...
        'BackgroundColor', [0 0 0],...
        'ForegroundColor', [1 1 1],...
        'BusyAction','queue',...
        'TooltipString','These values are offset from volume center, should be toward +X',...
        'String','x,y,z');
	
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	handles.flip = uicontrol('Parent',GUI,'Style','pushbutton','Units','Normalized', Font, ...
        'Position',[.55 .22 .10 .03], 'HorizontalAlignment', 'left',...
        'BackgroundColor', [.3 .3 .3],'ForegroundColor', [1 1 1],...
        'BusyAction','queue',...
        'TooltipString','Flip Left and Right (viewer only, see also avw_flip).',...
        'String','Flip L/R',...
        'Callback','avw_view_truecolor([],[],''flip'');');
	handles.flipStatus = uicontrol('Parent',GUI,'Style','text','Units','Normalized', Font, ...
        'Position',[.65 .22 .15 .03], 'HorizontalAlignment', 'right',...
        'BackgroundColor', [0 0 0],...
        'ForegroundColor', [1 1 1],...
        'BusyAction','queue',...
        'TooltipString','Flipped Status',...
        'String','orig');
	
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	handles.histogram = uicontrol('Parent',GUI,'Style','pushbutton','Units','Normalized', Font, ...
        'Position',[.85 .46 .10 .03], 'HorizontalAlignment', 'left',...
        'BackgroundColor', [.3 .3 .3],'ForegroundColor', [1 1 1],...
        'BusyAction','queue',...
        'TooltipString','Histogram of Volume Intensity',...
        'String','Histogram',...
        'Callback','avw_view_truecolor([],[],''histogram'');');
	
    
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	handles.coord = uicontrol('Parent',GUI,'Style','popupmenu','Units','Normalized', Font, ...
        'Position',[.85 .42 .10 .03], 'HorizontalAlignment', 'left',...
        'BackgroundColor', [.3 .3 .3],'ForegroundColor', [1 1 1],...
        'BusyAction','queue',...
        'TooltipString','Voxel or Mensurated Axis Coordinates',...
        'String',{'Voxels','mm','meters'},...
        'Callback','avw_view_truecolor([],[],''coordinates'');');
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	handles.dimmer = uicontrol('Parent',GUI,'Style','pushbutton','Units','Normalized', Font, ...
        'Position',[.55 .01 .10 .03], 'HorizontalAlignment', 'left',...
        'BackgroundColor', [.3 .3 .3],'ForegroundColor', [1 1 1],...
        'BusyAction','queue',...
        'TooltipString','Dim by 1%',...
        'String','Dimmer',...
        'Callback','avw_view_truecolor([],[],''dimmer'');');
	
	handles.brighter = uicontrol('Parent',GUI,'Style','pushbutton','Units','Normalized', Font, ...
        'Position',[.65 .01 .10 .03], 'HorizontalAlignment', 'left',...
        'BackgroundColor', [.3 .3 .3],'ForegroundColor', [1 1 1],...
        'BusyAction','queue',...
        'TooltipString','Brighten by 1%',...
        'String','Brighter',...
        'Callback','avw_view_truecolor([],[],''brighter'');');
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	Font.FontWeight = 'bold';
	
	% View avw.hdr
	handles.Bhdr = uicontrol('Parent',GUI,'Style','pushbutton','Units','Normalized', Font, ...
        'Position',[.8 .01 .08 .03],...
        'String','HDR','BusyAction','queue',...
        'TooltipString','View the .hdr parameters.',...
        'BackgroundColor',[0.0 0.0 0.5],...
        'ForegroundColor',[1 1 1], 'HorizontalAlignment', 'center',...
        'Callback',strcat('AVWVIEW = get(gcbf,''Userdata''); ',...
        'avw_view_hdr(AVWVIEW.avw,AVWVIEW.gui);',...
        'clear AVWVIEW;'));
	
	% OK: Return the avw!
	handles.Bquit = uicontrol('Parent',GUI,'Style','pushbutton','Units','Normalized', Font, ...
        'Position',[.9 .01 .08 .03],...
        'String','RETURN','BusyAction','queue',...
        'BackgroundColor',[0.0 0.5 0.0],...
        'ForegroundColor', [1 1 1], 'HorizontalAlignment', 'center',...
        'Callback','avw_view_truecolor([],[],''quit'');');
	
	% Update the gui_struct handles for this gui
	if exist('parent','var'), AVWVIEW.parent.gui = parent; end
	AVWVIEW.avw = avw;
	AVWVIEW.handles = handles;
	set(AVWVIEW.gui,'Userdata',AVWVIEW);
	set(AVWVIEW.gui,'HandleVisibility','callback');
	
return


